@@ -156,10 +156,11 @@ def create_oidc_client(kca, id_broker_fqdn, authority, secret):
     kc_client_id = kca.get_client_id(client_config["clientId"])
     scopes = kca.get_client_scopes()
     for scope in scopes:
-        if scope["name"] in ["profile", "email"]:
-            kca.connection.raw_delete(
+        if scope["name"] == "profile":
+            kca.raw_delete(
                 f"/admin/realms/ucs/clients/{kc_client_id}/default-client-scopes/{scope['id']}"
             )
+            break
     kca.add_mapper_to_client(kc_client_id, entryUUID_mapper)
     print("OIDC client has been created", file=sys.stderr)
 
