@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2022 Univention GmbH
 #
 # http://www.univention.de/
@@ -27,6 +26,7 @@
 # /usr/share/common-licenses/AGPL-3; if not, see
 # <http://www.gnu.org/licenses/>.
 
+
 import logging
 import os
 from typing import Any, Dict, Iterable, List, Union
@@ -75,9 +75,16 @@ from ucsschool_id_connector.models import (
     ListenerUserAddModifyObject,
     ListenerUserRemoveObject,
     SchoolAuthorityConfiguration,
+    SchoolUserRole,
+    UnknownSchoolUserRole,
 )
 from ucsschool_id_connector.plugins import hook_impl, plugin_manager
-from ucsschool_id_connector.utils import school_class_dn_regex, workgroup_dn_regex
+from ucsschool_id_connector.utils import (
+    legal_guardian_dn_regex,
+    school_class_dn_regex,
+    user_dn_regex,
+    workgroup_dn_regex,
+)
 
 ContextDictType = Dict[str, Dict[str, Iterable[str]]]
 
@@ -209,11 +216,15 @@ class IDBrokerPerSAUserDispatcher(PerSchoolAuthorityUserDispatcherBase):
         super(IDBrokerPerSAUserDispatcher, self).__init__(school_authority, plugin_name)
         self.class_dn_regex = school_class_dn_regex()
         self.workgroup_dn_regex = workgroup_dn_regex()
+        self.legal_guardian_dn_regex = legal_guardian_dn_regex()
+        self.user_dn_regex = user_dn_regex()
         self.attribute_mapping = {
             "id": "id",
             "username": "user_name",
             "firstname": "first_name",
             "lastname": "last_name",
+            "legal_guardians": "legal_guardians",
+            "legal_wards": "legal_wards",
             "context": "context",
         }
         self.id_broker_school_class = IDBrokerSchoolClass(self.school_authority, "id_broker")
@@ -261,6 +272,42 @@ class IDBrokerPerSAUserDispatcher(PerSchoolAuthorityUserDispatcherBase):
     async def _handle_attr_id(self, obj: ListenerUserAddModifyObject) -> str:
         return obj.id
 
+    async def _handle_attr_legal_guardians(self, obj: ListenerUserAddModifyObject) -> list[str]:
+        record_uids = []
+        for dn in obj.legal_guardians:
+            user_entries = await self.ldap_access.search(
+                base=dn,
+                filter_s="(objectClass=ucsschoolLegalGuardian)",
+                attributes=["entryUUID"],
+            )
+            if len(user_entries) < 1:
+                self.logger.warning(f"Legal guardian {dn} not found.")
+                continue
+            entry_uuid = str(user_entries[0].entryUUID)
+            if not await self.id_broker_user.exists(entry_uuid):
+                # Connection between users will be fixed when the legal guardian will be synced
+                self.logger.debug(f"Legal guardian {dn} not yet synced to ID Broker.")
+                continue
+            record_uids.append(entry_uuid)
+        return sorted(record_uids)
+
+    async def _handle_attr_legal_wards(self, obj: ListenerUserAddModifyObject) -> list[str]:
+        record_uids = []
+        for dn in obj.legal_wards:
+            user_entries = await self.ldap_access.search(
+                base=dn, filter_s="(objectClass=ucsschoolStudent)", attributes=["entryUUID"]
+            )
+            if len(user_entries) < 1:
+                self.logger.warning(f"Legal ward {dn} not found.")
+                continue
+            entry_uuid = str(user_entries[0].entryUUID)
+            if not await self.id_broker_user.exists(entry_uuid):
+                # Connection between users will be fixed when the legal ward will be synced
+                self.logger.debug(f"Legal ward {dn} not yet synced to ID Broker.")
+                continue
+            record_uids.append(entry_uuid)
+        return sorted(record_uids)
+
     async def search_params(
         self, obj: Union[ListenerUserAddModifyObject, ListenerUserRemoveObject]
     ) -> Dict[str, Any]:
@@ -291,6 +338,54 @@ class IDBrokerPerSAUserDispatcher(PerSchoolAuthorityUserDispatcherBase):
                     work_group, school, self.ldap_access, self.id_broker_work_group, self.logger
                 )
 
+    async def _school_context_from_ldap_entry(self, ldap_entry):
+        """Generate a school context from an ldap entry
+
+        The ldap entry has to contain attributes objectClass and ucsschoolSchool
+        """
+        user_dn_match = self.user_dn_regex.match(ldap_entry.entry_dn)
+        if not user_dn_match:
+            self.logger.error(f"dn {ldap_entry.entry_dn!r} does not match {self.user_dn_regex!r}")
+            return None
+
+        schools = sorted(ldap_entry.ucsschoolSchool)
+        primary_school = user_dn_match.groupdict()["ou"]
+        schools.remove(primary_school)
+        schools.insert(0, primary_school)
+
+        # At the moment we should _only_ sync students and legal guardians -> users_entry_uuids
+        # should only contain users of these types
+        options = set(ldap_entry.objectClass)
+        if "ucsschoolStudent" in options:
+            roles = [SchoolUserRole.student]
+        elif "ucsschoolLegalGuardian" in options:
+            roles = [SchoolUserRole.legal_guardian]
+        else:
+            raise UnknownSchoolUserRole(
+                f"Unknown or missing school user type in options: {self.options!r}"
+            )
+
+        context = {
+            school_name: {"roles": roles, "classes": set(), "workgroups": set()}
+            for school_name in schools
+        }
+        if ldap_entry.memberOf:
+            for group_dn in ldap_entry.memberOf:
+
+                user_dn_match = self.class_dn_regex.match(group_dn)
+                if user_dn_match:
+                    name = user_dn_match.groupdict()["name"]
+                    school = user_dn_match.groupdict()["ou"]
+                    context[school]["classes"].add(name)
+                    continue
+
+                user_dn_match = self.workgroup_dn_regex.match(group_dn)
+                if user_dn_match:
+                    name = user_dn_match.groupdict()["name"]
+                    school = user_dn_match.groupdict()["ou"]
+                    context[school]["workgroups"].add(name)
+        return context
+
     def _empty_groups(self, context: ContextDictType) -> ContextDictType:
         self.logger.debug("Empty classes of user for faster initial import mode.")
         for school in context:
@@ -298,6 +393,9 @@ class IDBrokerPerSAUserDispatcher(PerSchoolAuthorityUserDispatcherBase):
             context[school]["workgroups"] = []
         return context
 
+    async def _create_missing_objects(self, request_body: Dict[str, Any]):
+        await self._create_schools_and_school_groups_if_missing(request_body["context"])
+
     async def do_create(self, request_body: Dict[str, Any]) -> None:
         """Create a user object at the target."""
         self.logger.info(
@@ -306,7 +404,7 @@ class IDBrokerPerSAUserDispatcher(PerSchoolAuthorityUserDispatcherBase):
         self.logger.debug("request_body=%r", request_body)
         if self.initial_import_mode:
             request_body["context"] = self._empty_groups(request_body["context"])
-        await self._create_schools_and_school_groups_if_missing(request_body["context"])
+        await self._create_missing_objects(request_body)
         user: User = await self.id_broker_user.create(User(**request_body))
         self.logger.info("User created: %r.", user)
 
@@ -316,7 +414,9 @@ class IDBrokerPerSAUserDispatcher(PerSchoolAuthorityUserDispatcherBase):
         self.logger.debug("request_body=%r", request_body)
         if self.initial_import_mode:
             request_body["context"] = self._empty_groups(request_body["context"])
-        await self._create_schools_and_school_groups_if_missing(request_body["context"])
+            request_body["legal_wards"] = []
+            request_body["legal_guardians"] = []
+        await self._create_missing_objects(request_body)
         user: User = await self.id_broker_user.update(User(**request_body))
         self.logger.info("User modified: %r.", user)
 
