@@ -47,6 +47,7 @@ from ucsschool.kelvin.client import (
     WorkGroupResource as KelvinWorkGroupResource,
 )
 from ucsschool_id_connector.ldap_access import LDAPAccess
+from ucsschool_id_connector.user_scheduler import UserScheduler
 
 # load ID Broker plugin
 ucsschool_id_connector.plugin_loader.load_plugins()
@@ -153,7 +154,7 @@ async def test_workgroup_exists_after_user_with_non_existing_workgroup_is_synced
 @pytest.mark.parametrize(
     "initial_import_mode", ["unset", True, False], ids=lambda x: f"initial_import_mode={x}"
 )
-@pytest.mark.parametrize("role", ["student", "teacher"])
+@pytest.mark.parametrize("role", ["student", "teacher", "legal_guardian"])
 @pytest.mark.asyncio
 async def test_sync_user(
     school_wildcard,
@@ -192,12 +193,25 @@ async def test_sync_user(
     )
     s_a_name = school_authority.name
     sender_user: Dict[str, Any] = await make_sender_user_special(
-        ous=[sender_school_1.name, sender_school_2.name], s_a_name=s_a_name, roles=(role,)
+        ous=[sender_school_1.name, sender_school_2.name],
+        s_a_name=s_a_name,
+        roles=(role,),
+        create_related_users=True,
     )
     remote_user: KelvinUser = await wait_for_kelvin_user_on_id_broker(
         s_a_name=s_a_name, school=sender_school_1.name, name=sender_user["name"]
     )
     assert sorted(f"{s_a_name}-{s}" for s in sender_user["schools"]) == sorted(remote_user.schools)
+
+    if role == "legal_guardian":
+        assert set(remote_user.legal_wards) == set(
+            [f"{s_a_name}-{s}" for s in sender_user["legal_wards"]]
+        )
+    elif role == "student":
+        assert set(remote_user.legal_guardians) == set(
+            [f"{s_a_name}-{s}" for s in sender_user["legal_guardians"]]
+        )
+
     if initial_import_mode == 1:
         assert remote_user.school_classes == {}
     else:
@@ -216,7 +230,7 @@ async def test_sync_user(
 @pytest.mark.parametrize(
     "initial_import_mode", ["unset", True, False], ids=lambda x: f"initial_import_mode={x}"
 )
-@pytest.mark.parametrize("role", ["student", "teacher"])
+@pytest.mark.parametrize("role", ["student", "teacher", "legal_guardian"])
 @pytest.mark.asyncio
 async def test_sync_only_user_with_configured_school(
     school_wildcard,
@@ -290,7 +304,7 @@ async def test_sync_only_user_with_configured_school(
 @pytest.mark.parametrize(
     "initial_import_mode", ["unset", True, False], ids=lambda x: f"initial_import_mode={x}"
 )
-@pytest.mark.parametrize("role", ["student", "teacher"])
+@pytest.mark.parametrize("role", ["student", "teacher", "legal_guardian"])
 @pytest.mark.asyncio
 async def test_remove_user_after_last_configured_school_is_removed(
     kelvin_schools_on_sender,
@@ -861,3 +875,232 @@ async def test_group_is_updated_with_missing_users(
         await asyncio.sleep(2)
     else:
         raise AssertionError("Members not as expected.")
+
+
+@pytest.mark.asyncio
+async def test_legal_guardian_missing_users(
+    kelvin_school_on_sender,
+    wait_for_kelvin_user_on_id_broker,
+    make_sender_user_special,
+    schedule_schools_delete,
+    set_school_authority_on_id_connector,
+):
+    """Create some students, sync them, then remove two of them on the ID Broker side
+    and create a legal guardian on the sender side which has all of them as legal wards
+    """
+    school_authority = await set_school_authority_on_id_connector(schools=[kelvin_school_on_sender.name])
+    await schedule_schools_delete(
+        school_authority=school_authority, schools=[kelvin_school_on_sender.name]
+    )
+    s_a_name = school_authority.name
+    students: list[dict[str, Any]] = [
+        await make_sender_user_special(ous=[kelvin_school_on_sender.name], roles=("student",))
+        for _ in range(3)
+    ]
+    for sender_user in students:
+        await wait_for_kelvin_user_on_id_broker(
+            s_a_name=s_a_name, name=sender_user["name"], school=kelvin_school_on_sender.name
+        )
+    entry_uuids = []
+    ldap_access = LDAPAccess()
+    for student in students:
+        results = await ldap_access.search(
+            filter_s=f"(&(objectClass=posixAccount)(uid={student['name']}))",
+            attributes=["entryUUID"],
+        )
+        assert len(results) == 1
+        entry_uuids.append(results[0]["entryUUID"].value)
+
+    id_broker_user = IDBrokerUser(school_authority, "id_broker")
+
+    for i in range(2):
+        await id_broker_user.delete(entry_uuids[i])
+        await wait_for_kelvin_user_on_id_broker(
+            s_a_name=s_a_name,
+            name=students[i]["name"],
+            school=kelvin_school_on_sender.name,
+            must_exist=False,
+        )
+        assert not await id_broker_user.exists(entry_uuids[i])
+    legal_guardian = await make_sender_user_special(
+        ous=[kelvin_school_on_sender.name],
+        roles=("legal_guardian",),
+        legal_wards=[student["name"] for student in students],
+    )
+    await wait_for_kelvin_user_on_id_broker(
+        s_a_name=s_a_name,
+        name=legal_guardian["name"],
+        school=kelvin_school_on_sender.name,
+        must_exist=True,
+    )
+    results = await ldap_access.search(
+        filter_s=f"(&(objectClass=posixAccount)(uid={legal_guardian['name']}))",
+        attributes=["entryUUID"],
+    )
+    assert len(results) == 1
+    legal_guardian_entry_uuid = results[0]["entryUUID"].value
+    legal_guardian_on_id_broker = await id_broker_user.get(legal_guardian_entry_uuid)
+    assert set(legal_guardian_on_id_broker.legal_wards) == set(entry_uuids[2:])
+    scheduler = UserScheduler()
+    for i in range(2):
+        await scheduler.queue_user(students[i]["name"])
+        await wait_for_kelvin_user_on_id_broker(
+            s_a_name=s_a_name,
+            name=students[i]["name"],
+            school=kelvin_school_on_sender.name,
+        )
+        assert await id_broker_user.exists(entry_uuids[i])
+    legal_guardian_on_id_broker = await id_broker_user.get(legal_guardian_entry_uuid)
+    assert set(legal_guardian_on_id_broker.legal_wards) == set(entry_uuids)
+
+
+@pytest.mark.asyncio
+async def test_legal_ward_missing_users(
+    kelvin_school_on_sender,
+    wait_for_kelvin_user_on_id_broker,
+    make_sender_user_special,
+    schedule_schools_delete,
+    set_school_authority_on_id_connector,
+):
+    """Create some legal wards, sync them, then remove two of them on the ID Broker side
+    and create a legal ward on the sender side which has all of them as legal guards
+    """
+    school_authority = await set_school_authority_on_id_connector(schools=[kelvin_school_on_sender.name])
+    await schedule_schools_delete(
+        school_authority=school_authority, schools=[kelvin_school_on_sender.name]
+    )
+    s_a_name = school_authority.name
+    legal_guardians: list[dict[str, Any]] = [
+        await make_sender_user_special(ous=[kelvin_school_on_sender.name], roles=("legal_guardian",))
+        for _ in range(3)
+    ]
+    for sender_user in legal_guardians:
+        await wait_for_kelvin_user_on_id_broker(
+            s_a_name=s_a_name, name=sender_user["name"], school=kelvin_school_on_sender.name
+        )
+    entry_uuids = []
+    ldap_access = LDAPAccess()
+    for legal_guardian in legal_guardians:
+        results = await ldap_access.search(
+            filter_s=f"(&(objectClass=posixAccount)(uid={legal_guardian['name']}))",
+            attributes=["entryUUID"],
+        )
+        assert len(results) == 1
+        entry_uuids.append(results[0]["entryUUID"].value)
+
+    id_broker_user = IDBrokerUser(school_authority, "id_broker")
+
+    for i in range(2):
+        await id_broker_user.delete(entry_uuids[i])
+        await wait_for_kelvin_user_on_id_broker(
+            s_a_name=s_a_name,
+            name=legal_guardians[i]["name"],
+            school=kelvin_school_on_sender.name,
+            must_exist=False,
+        )
+        assert not await id_broker_user.exists(entry_uuids[i])
+    student = await make_sender_user_special(
+        ous=[kelvin_school_on_sender.name],
+        roles=("student",),
+        legal_guardians=[legal_guardian["name"] for legal_guardian in legal_guardians],
+    )
+    await wait_for_kelvin_user_on_id_broker(
+        s_a_name=s_a_name,
+        name=student["name"],
+        school=kelvin_school_on_sender.name,
+        must_exist=True,
+    )
+    results = await ldap_access.search(
+        filter_s=f"(&(objectClass=posixAccount)(uid={student['name']}))",
+        attributes=["entryUUID"],
+    )
+    assert len(results) == 1
+    student_entry_uuid = results[0]["entryUUID"].value
+    student_entry_uuid_on_id_broker = await id_broker_user.get(student_entry_uuid)
+    assert set(student_entry_uuid_on_id_broker.legal_guardians) == set(entry_uuids[2:])
+    scheduler = UserScheduler()
+    for i in range(2):
+        await scheduler.queue_user(legal_guardians[i]["name"])
+        await wait_for_kelvin_user_on_id_broker(
+            s_a_name=s_a_name,
+            name=legal_guardians[i]["name"],
+            school=kelvin_school_on_sender.name,
+        )
+        assert await id_broker_user.exists(entry_uuids[i])
+    student_on_id_broker = await id_broker_user.get(student_entry_uuid)
+    assert set(student_on_id_broker.legal_guardians) == set(entry_uuids)
+
+
+@pytest.mark.parametrize("school_wildcard", ("none", "partial"), ids=lambda x: f"school_wildcard={x}")
+@pytest.mark.asyncio
+async def test_sync_only_legal_role_with_configured_school(
+    school_wildcard,
+    kelvin_schools_on_sender,
+    wait_for_kelvin_user_on_id_broker,
+    id_broker_school_auth_conf,
+    make_sender_user_special,
+    schedule_schools_delete,
+    set_school_authority_on_id_connector,
+):
+    """
+    sender_legal_guardian1 should never be created
+    sender_legal_ward1 should never be created
+    These users should be filtered from the legal_guardians and legal_wards attribute
+    """
+    un_configured_sender_school, configured_sender_school = await kelvin_schools_on_sender(2)
+    if school_wildcard == "none":
+        school_filter = [configured_sender_school.name]
+    elif school_wildcard == "partial":
+        school_filter = [f"{configured_sender_school.name[0]}*{configured_sender_school.name[2:]}"]
+        if (
+            f"{configured_sender_school.name[0]}*{configured_sender_school.name[2:]}"
+            == f"{un_configured_sender_school.name[0]}*{un_configured_sender_school.name[2:]}"
+        ):
+            # How probable is this?
+            raise ValueError(
+                f"{configured_sender_school.name} and {un_configured_sender_school.name} are to similar"
+            )
+    else:
+        raise ValueError(f"Invalid school_wildcard {school_wildcard}")
+    school_authority = await set_school_authority_on_id_connector(schools=school_filter)
+    await schedule_schools_delete(
+        school_authority=school_authority,
+        schools=[configured_sender_school.name, un_configured_sender_school.name],
+    )
+    s_a_name = school_authority.name
+    sender_legal_guardian1: Dict[str, Any] = await make_sender_user_special(
+        ous=[un_configured_sender_school.name], s_a_name=s_a_name, roles=("legal_guardian",)
+    )
+    sender_legal_guardian2: Dict[str, Any] = await make_sender_user_special(
+        ous=[configured_sender_school.name], s_a_name=s_a_name, roles=("legal_guardian",)
+    )
+    # sender_legal_ward1
+    await make_sender_user_special(
+        ous=[un_configured_sender_school.name],
+        s_a_name=s_a_name,
+        roles=("student",),
+        legal_guardians=[sender_legal_guardian1["name"], sender_legal_guardian2["name"]],
+    )
+    sender_legal_ward2: Dict[str, Any] = await make_sender_user_special(
+        ous=[configured_sender_school.name],
+        s_a_name=s_a_name,
+        roles=("student",),
+        legal_guardians=[sender_legal_guardian1["name"], sender_legal_guardian2["name"]],
+    )
+    for attempt in Retrying(
+        reraise=True,
+        stop=stop_after_attempt(3),
+        retry=retry_if_exception_type(AssertionError),
+        wait=wait_fixed(5),
+    ):
+        with attempt:
+            remote_legal_guardian2: KelvinUser = await wait_for_kelvin_user_on_id_broker(
+                s_a_name=s_a_name,
+                school=configured_sender_school.name,
+                name=sender_legal_guardian2["name"],
+            )
+            remote_legal_ward2: KelvinUser = await wait_for_kelvin_user_on_id_broker(
+                s_a_name=s_a_name, school=configured_sender_school.name, name=sender_legal_ward2["name"]
+            )
+            assert remote_legal_guardian2.legal_wards == [f"{s_a_name}-{sender_legal_ward2['name']}"]
+            assert remote_legal_ward2.legal_guardians == [f"{s_a_name}-{sender_legal_guardian2['name']}"]
